// Copyright 2021 Esri
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import ArcGIS

class ViewController: UIViewController {
    @IBOutlet weak var sceneView: AGSSceneView!

    override func viewDidLoad() {
        super.viewDidLoad()
        setupScene()

        setupUI()

        setupAnalysisOverlay()

    }

    // MARK: Scene

    private func setupScene() {
        let scene: AGSScene = {
            let portal = AGSPortal.arcGISOnline(withLoginRequired: false)

            let item = AGSPortalItem(portal: portal, itemID: "7558ee942b2547019f66885c44d4f0b1")

            return AGSScene(item: item)
        }()
        sceneView.scene = scene

        sceneView.touchDelegate = self

    }

    // MARK: UI

    private lazy var slider: UISlider = {
        let slider = UISlider()
        slider.maximumValue = 12_000
        slider.minimumValue = 1_000
        slider.value = 3_000
        slider.addTarget(self, action: #selector(sliderValueDidChange), for: .allTouchEvents)
        return slider
    }()

    private func setupUI() {
        navigationItem.titleView = slider
        navigationItem.rightBarButtonItem = UIBarButtonItem(
            title: "Clear",
            style: .plain,
            target: self,
            action: #selector(clearScene)
        )
    }

    @objc
    func sliderValueDidChange(_ sender: UISlider) {
        viewshed.maxDistance = Double(sender.value)
    }

    @objc
    func clearScene(_ sender: UIBarButtonItem) {
        clearViewshed()
    }

    // MARK: Viewshed Analysis

    private lazy var viewshed: AGSLocationViewshed = {
        let viewshed = AGSLocationViewshed(
            location: AGSPointMake3D(0, 0, 0, 0, .wgs84()),
            heading: 0,
            pitch: 90,
            horizontalAngle: 360,
            verticalAngle: 180,
            minDistance: 10,
            maxDistance: Double(slider.value)
        )
        viewshed.isVisible = false
        return viewshed
    }()

    private func setupAnalysisOverlay() {
        let analysisOverlay = AGSAnalysisOverlay()
        sceneView.analysisOverlays.add(analysisOverlay)
        analysisOverlay.analyses.add(viewshed)
    }

    private func setViewshedLocation(point: AGSPoint) {
        viewshed.location = point
        if !viewshed.isVisible {
            viewshed.isVisible = true
        }
    }

    private func clearViewshed() {
        viewshed.isVisible = false
    }

}

extension ViewController: AGSGeoViewTouchDelegate {

    func geoView(_ geoView: AGSGeoView, didLongPressAtScreenPoint screenPoint: CGPoint, mapPoint: AGSPoint) {

        setViewshedLocation(point: mapPoint)

    }

    func geoView(_ geoView: AGSGeoView, didMoveLongPressToScreenPoint screenPoint: CGPoint, mapPoint: AGSPoint) {

        setViewshedLocation(point: mapPoint)

    }

    func geoView(_ geoView: AGSGeoView, didEndLongPressAtScreenPoint screenPoint: CGPoint, mapPoint: AGSPoint) {

        setViewshedLocation(point: mapPoint)

    }

    func geoViewDidCancelLongPress(_ geoView: AGSGeoView) {

        clearViewshed()

    }

}


